#!/bin/bash

question_select()
{
    # $1 title
    # $2 value

    wish <<EOF
wm title . "$1"

label .l -text "$2" -padx 5 -pady 5
pack .l

button .ok -text "Ok" -command {exit 0}
button .cancel -text "Cancel" -command {exit 1}
pack configure .ok .cancel -side left -anchor sw -padx 10 -pady 5
EOF
    echo $?
}

entry_hide()
{
    # $1 -> title                                                            
    # $2 -> text

    wish <<EOF
wm title . "$1"

label .l -text "$2" -padx 5 -pady 5
pack .l

entry .e -show "*" -textvariable entree
set entree "$3"
pack .e -fill both

button .ok -text "Ok" -command {puts [.e get]; exit 0}
button .cancel -text "Cancel" -command {exit 1}
pack configure .ok .cancel -side left -anchor sw -padx 10 -pady 5
EOF
    return $?
}

entry_text()
{
    # $1 -> title
    # $2 -> text
    # $3 -> default entry

    wish <<EOF
wm title . "$1"

label .l -text "$2" -padx 5 -pady 5
pack .l

entry .e -textvariable entree
set entree "$3"
pack .e -fill both

button .ok -text "Ok" -command {puts [.e get]; exit 0}
button .cancel -text "Cancel" -command {exit 1}
pack configure .ok .cancel -side left -anchor sw -padx 10 -pady 5
EOF
    return $?
}


checklist_select()
{
    # $1 title
    # $2 text
    # $3 values. Must recreate data
    
    var=""
    SAVE_IFS="$IFS"
    IFS="
"
    init_wish=""
    cpt=1
    for line in $3 ; do
	
	var="`echo \"$line\" | awk 'BEGIN {var=\"\"} { for (i = 2; i <= NF; i++) if (var) var=var \" \" $i; else var=$i} END {print var }'`"
	var="checkbutton .c$cpt -text $var -variable Statebutton$cpt
pack .c$cpt -anchor w
lappend list_check \".c$cpt\"
"
	if [ `echo "$line" | awk '{print $1}'` = "TRUE" ] ; then
	    var="$var
.c$cpt select
"
	fi
	init_wish="$init_wish $var"
	cpt=$(($cpt + 1))
    done
    IFS=$SAVE_IFS

    wish <<EOF

proc aff_checking {args} {
    global list_check

    set aff_values ""
    foreach item \$list_check {
       set name [\$item cget -variable] 
       global \$name
       eval set value \\\$\$name
       if {\$value == "1"} {
          if {\$aff_values != ""} {
             set aff_values "\$aff_values|[\$item cget -text]"
          } else {
             set aff_values [\$item cget -text]
          }
       }
    }
    puts \$aff_values
}

set list_check ""
wm title . "$1"

label .l -text "$2" -padx 5 -pady 5
pack .l
$init_wish

button .ok -text "Ok" -command {aff_checking ; exit 0}
button .cancel -text "Cancel" -command {exit 1}
pack configure .ok .cancel -side left -anchor e -padx 10 -pady 5
EOF
    return $?
}

radiolist_select()
{
    # $1 title
    # $2 text
    # $3 values. Must recreate data
    # $4 force height
    # $5 force width

    var=""
    SAVE_IFS="$IFS"
    IFS="
"
    init_wish=""
    cpt=1
    for line in $3 ; do

	var="`echo \"$line\" | awk 'BEGIN {var=\"\"} { for (i = 2; i <= NF; i++) if (var) var=var \" \" $i; else var=$i} END {print var }'`"
	var="radiobutton .r$cpt -text $var -variable state_button -value $cpt
pack .r$cpt -anchor w
"
	if [ `echo "$line" | awk '{print $1}'` = "TRUE" ] ; then
	    var="$var
.r$cpt select
"
	fi
	init_wish="$init_wish $var"
	cpt=$(($cpt + 1))
    done
    IFS=$SAVE_IFS

    wish <<EOF
wm title . "$1"

label .l -text "$2" -padx 5 -pady 5
pack .l
$init_wish

button .ok -text "Ok" -command {puts [.r\$state_button cget -text]; exit 0}
button .cancel -text "Cancel" -command {exit 1}
pack configure .ok .cancel -side left -anchor e -padx 10 -pady 5
EOF
    return $?
}

info_text()
{
    # $1 -> title                                                                  # $2 -> text                                                                
    wish <<EOF
wm title . "$1"
label .l -text "$2" -padx 5 -pady 5
pack .l

button .back -text "Ok" -command {exit 0}
pack configure .back -anchor e -padx 10 -pady 5
EOF
    return $?
}

files_selection() 
{
    wish << EOF
option add *Font {Helvetica 11 bold}

set bgcldefault_text "grey"
set fgcldefault_text "black"
set bgcldefault_select "steelblue"
set selectedlist ""

proc remove_in_array {arg} {
    
    global selectedlist

    set tmp_list ""
    if {[pwd] == "/" } {
	set path_good "/"
    } else {
	set path_good "[pwd]/"
    }
    foreach item [split \$selectedlist "|" ] {
	if {\$item != "\$path_good\$arg"} {
	    if {\$tmp_list != ""} {
		set tmp_list "\$tmp_list|\$item"
	    } else {
		set tmp_list "\$item"
	    }
	}
    }
    set selectedlist "\$tmp_list"
}

proc get_files {directory} {

    global bgcldefault_text
    global fgcldefault_text
    global bgcldefault_select
    global dir_img
    global file_img
    
    global path
    
    global markext
    global markedlist 
    global selectedlist
    global lastselect
    global list_file_sort
    global list_directory
    global right
    global tagarray
    global sorttype
    global viewright
    
    set markedlist   ""
    set lastselect   0

    if "[file isdirectory \$directory]" then {
	if "[file readable \$directory]" then {   
	    cd \$directory
	    set path [pwd]
	    if {\$path == "/" } {
		set path_good "/"
	    } else {
		set path_good "\$path/"
	    }
	    set bufferpath \$path
	    
	    # Nettoie le text precedent
	    .list.text configure -state normal
	    .list.text delete 1.0 end
	    
	    set fields_split_selected [split \$selectedlist "|"]
	    set list_link ""
	    set list_directory ""
	    set list_file ""
	    
	    if {\$sorttype == "date"} {
		set cmd "-1at"
	    } else {
		set cmd "-1a"
	    }
	    
	    foreach item [split [exec ls \$cmd] "\n"] {
		set type [file type "\$item"]
		if {\$type == "link"} {
		    set tmp_item "\$item"
		    while {1} {
			set target [file readlink "\$tmp_item"]
			
			#### REUSSIR A TESTER SI TARGET EST UN LIEN
			# file exists renvoit 0 :(

			if {[file exists "\$target"] == 0} {
			    set type "NULL"
			    break
			}
			if {[file type "\$target"] != "link"} {
			    set type [file type "\$target"]
			    break
			}
			set tmp_item "\$target"
		    }
		    if {\$type == "NULL"} {
			continue
		    }
		}

		eval set tmp \\\$list_\$type
                if { \$tmp != "" } {
		    set list_\$type "\$tmp|\$item"
		} else {
		    set list_\$type "\$item"
		}
		if {\$viewright} { 
		    set lsline [exec ls -ld "\$item"] 
		    set right(\$item) "[lindex \$lsline 2] [lindex \$lsline 3] [lindex \$lsline 0]\t"	 
		} else {
		    set right(\$item) ""
		}
	    }
	    set numd    -2
	    set numf    0
 
	    set choice ""

	    foreach item [ split \$list_directory "|" ] {
		set date [clock format [file mtime \$item] -format "%d %B %Y" ]
		incr numd
		.list.text image create end -image \$dir_img
		.list.text insert end "\$item\t\t\$right(\$item)\$date\n" dir\$numd
		.list.text tag configure dir\$numd -foreground \$fgcldefault_text -background \$bgcldefault_text
		.list.text tag bind dir\$numd <Enter> ".list.text tag configure dir\$numd -background yellow -foreground red"
		.list.text tag bind dir\$numd <Leave> ".list.text tag configure dir\$numd -background \$bgcldefault_text -foreground \$fgcldefault_text"
		.list.text tag bind dir\$numd <Double-ButtonPress-1> "remove_in_array \$item; get_files \$item"
		set tagarray(\$item) "dir\$numd"
		.list.text tag bind dir\$numd <ButtonPress-1>        "set lastselect \$numd ; select_deselect \$item"
		.list.text tag bind dir\$numd <Shift-ButtonPress-1>  "select_list dir \$numd"  
		if {[lsearch \$fields_split_selected "\$path_good\$item"] >= 0} {
		    .list.text tag configure dir\$numd -background \$bgcldefault_select -foreground black
		    .list.text tag bind dir\$numd <Leave> ".list.text tag configure dir\$numd -background \$bgcldefault_select -foreground black"
		}
	    }
	    
	    set list_sortsize ""
	    set cpt 0
	    
	    foreach item [ split \$list_file "|" ] {
		incr cpt
		set size [file size \$item]
		set ident(\$size.\$cpt) \$item
	    }
   
	    if {\$sorttype=="size"} {
		foreach key [lsort -decreasing -real [array names ident] ]  { 
		    lappend list_sortsize \$ident(\$key) 
		}
		set list_file_sort \$list_sortsize
	    } else {
		set list_file_sort \$list_file
	    } 
   
	    foreach item [ split \$list_file_sort "|" ] {
		incr numf
		set date [clock format [file mtime \$item] -format "%d %B %Y" ]
		set size [file size \$item]
		.list.text image create end -image \$file_img
		.list.text insert end "\$item\t\$size\t\$right(\$item)\$date\n" fic\$numf
		if {[file extension \$item] == ".\$markext"} {
		    lappend markedlist \$item
		    .list.text tag configure fic\$numf -foreground yellow -background \$bgcldefault_text
		    .list.text tag bind fic\$numf <Leave> ".list.text tag configure fic\$numf -background \$bgcldefault_text -foreground yellow"
		} else {
		    .list.text tag configure fic\$numf -foreground \$fgcldefault_text -background \$bgcldefault_text
		    .list.text tag bind fic\$numf <Leave> ".list.text tag configure fic\$numf -background \$bgcldefault_text -foreground \$fgcldefault_text"
		}
		.list.text tag bind fic\$numf <Enter> ".list.text tag configure fic\$numf -background yellow -foreground red"        
		set tagarray(\$item) fic\$numf
		.list.text tag bind fic\$numf <ButtonPress-1> "set lastselect \$numf; select_deselect \$item"
		.list.text tag bind fic\$numf <Shift-ButtonPress-1> "select_list file \$numf"
		if {[lsearch \$fields_split_selected "\$path_good\$item"] >= 0} {
		    .list.text tag configure fic\$numf -background \$bgcldefault_select -foreground black
		    .list.text tag bind fic\$numf <Leave> ".list.text tag configure fic\$numf -background \$bgcldefault_select -foreground black"
		}
	    }   
	} else {
	    .summary configure -text "Probleme de permissions sur \$path/\$directory"
	}	
    } else {
	.summary configure -text "Mauvais chemin specifie - Merci d'en choisir un autre"
    }
    
    .list.text configure -foreground \$fgcldefault_text -background \$bgcldefault_text
    .list.text configure -selectbackground \$bgcldefault_text -selectforeground \$fgcldefault_text -selectborderwidth 0
    .list.text configure -state disable
}

proc select_list {lasttype index} { 
    
    global lastselect
    global list_file_sort
    global list_directory
    global noshift

    set noshift 0
    
    if { \$index > \$lastselect } {
	set maxindex \$index
	set minindex \$lastselect
    } else {
	set maxindex \$lastselect
	set minindex \$index
	incr minindex -1
	incr maxindex -1
    }
   
    switch \$lasttype {
	file {
	    set fields_split [ split \$list_file_sort "|" ]
	    for {set i \$minindex} {\$i<\$maxindex} {incr i} { 
		set tmp [ lindex \$fields_split \$i ]
		eval select_deselect \$tmp
	    } 
	}
	dir  {
	    incr minindex +2
	    incr maxindex +2
	    set fields_split [ split \$list_directory "|" ]
	    for {set i \$minindex} {\$i<\$maxindex} {incr i} { 
		set tmp [lindex \$fields_split \$i ]
		eval select_deselect \$tmp
	    }
	}
    }
    set noshift 1
}

proc select_deselect {args} {

    global bgcldefault_text
    global fgcldefault_text
    global bgcldefault_select

    global selectedlist
    global tagarray
    global noshift

    if {[pwd] == "/" } {
	set path_good "/"
    } else {
	set path_good "[pwd]/"
    }
    set tag \$tagarray(\$args)
    if {[lsearch [ split \$selectedlist "|" ] "\$path_good\$args"] < 0} then { 

	if { \$selectedlist != "" } {
	    set selectedlist "\$selectedlist|\$path_good\$args"
	} else {
	    set selectedlist "\$path_good\$args"
	}
	.list.text tag configure \$tag -background \$bgcldefault_select -foreground black
	.list.text tag bind \$tag <Leave> ".list.text tag configure \$tag -background \$bgcldefault_select -foreground black"      
    } elseif {\$noshift} {
	remove_in_array "\$args"
	switch [file type \$args] {
	    directory {
		.list.text tag configure \$tag -background \$bgcldefault_text -foreground \$fgcldefault_text
		.list.text tag bind \$tag <Leave> ".list.text tag configure \$tag -background \$bgcldefault_text -foreground \$fgcldefault_text"
	    }
	    file {
		.list.text tag configure \$tag -background \$bgcldefault_text -foreground \$fgcldefault_text
		.list.text tag bind \$tag <Leave> ".list.text tag configure \$tag -background \$bgcldefault_text -foreground \$fgcldefault_text"
	    }
	}
    }
}

set path \$env(PWD)
set markext mark

wm title . "$1"

set bg [. cget -bg]
set fg #000000
tk_setPalette \$bg
option add *background \$bg
option add *foreground \$fg
option add *activeBackground \$bg
option add *activeForeground \$fg
option add *selectcolor \$fg
option add *selectColor \$fg
option add *disabledforeground \$fg
option add *selectBackground \$fg
option add *selectForeground \$bg
	
frame .barmenu -relief raised -borderwidth 2
pack .barmenu -padx 4 -pady 4 -side top -anchor nw -fill x

frame .goto
pack  .goto -padx 4 -pady 4 -side top -fill x

label .goto.dirlabel -text "Goto Path :"
pack  .goto.dirlabel -side left -padx 4 -pady 4

entry .goto.dir -text path
pack  .goto.dir -side left -padx 4 -pady 4 -expand yes -fill x
bind  .goto.dir <KeyPress-Return> { get_files [.goto.dir get] }

button .goto.back -text "Ok" -command {puts "\$selectedlist"; exit 0}
pack   .goto.back -side left -padx 4 -pady 4

button .goto.home -text "Cancel" -command {exit 1}
pack   .goto.home -side left -padx 4 -pady 4
frame .extmark
pack  .extmark -padx 4 -pady 4 -fill x
label .extmark.extlabel -text "Extension a utiliser :"
pack  .extmark.extlabel -side left -padx 4 -pady 4

entry .extmark.extentry -text markext
pack  .extmark.extentry -side left -padx 4 -pady 4
bind  .extmark.extentry <KeyPress-Return> {get_files [.goto.dir get] }

label .summary
pack .summary -side top -anchor nw

frame .list
pack .list -expand yes -fill both -padx 4 -pady 4

scrollbar .list.s -command {.list.text yview} -orient vertical
pack .list.s -side right -fill y

text .list.text -yscrollcommand {.list.s set } -wrap none
pack .list.text -side left -expand yes -fill both

.list.text configure -tabs {10.0c right 5.0c right 18.0c left}


set dir_img [image create photo -file "./icons/dir.gif"]
set file_img [image create photo -file "./icons/f.gif"]


frame .bar
pack .bar -padx 4 -pady 4 -anchor nw

checkbutton .bar.sortdate -text "Tri par Date (Ctrl-d)" \
                          -variable sorttype -onvalue "date" -offvalue 0 \
			  -command { get_files [.goto.dir get] } 
pack .bar.sortdate -side left -padx 4 -pady 4

bind . <Control-KeyPress-d> { .bar.sortdate invoke }

checkbutton .bar.sortsize -text "Tri par Taille (Ctrl-t)" \
    -variable sorttype -onvalue "size" -offvalue 0 \
    -command { get_files [.goto.dir get] } 
pack .bar.sortsize -side left -padx 4 -pady 4

bind . <Control-KeyPress-t> { .bar.sortsize invoke }

checkbutton .bar.viewright -text "Voir Permissions (Ctrl-p)" \
    -variable viewright -onvalue 1 -offvalue 0 \
    -command { get_files [.goto.dir get] } 
pack .bar.viewright -side left -padx 4 -pady 4

bind . <Control-KeyPress-r> { .bar.viewright invoke }

set viewright 0
set noshift   1

get_files \$path
EOF
    return $?
}
